@@ -135,8 +135,6 @@ def release(jira_creds_file,
     auth_jira = JIRA(jira_options, oauth=jira_oauth_dict)
 
     github_token = read_github_creds(github_token_file)
-    if not github_token:
-        sys.exit(1)
     auth_gh = Github(github_token)
 
     if not is_valid_remote(remote):
@@ -367,7 +365,8 @@ def build_c_driver(c_driver_install_dir, c_driver_build_ref, quiet):
     env['PREFIX'] = mongoc_prefix
     if not c_driver_build_ref:
         c_driver_build_ref = 'master'
-    run_shell_script('./.evergreen/install_c_driver.sh ' + c_driver_build_ref, env=env)
+    mongocrypt_build_ref = 'master'
+    run_shell_script('MONGOC_VERSION=' + c_driver_build_ref + ' MONGOCRYPT_VERSION=' + mongocrypt_build_ref + ' ./.evergreen/install_c_driver.sh', env=env)
 
     if not quiet:
         click.echo('C Driver build was successful.')
@@ -526,8 +525,7 @@ def read_github_creds(github_token_file):
     Read the GitHub token from the specified file and return it as a string.
     """
 
-    # See https://github.blog/2021-04-05-behind-githubs-new-authentication-token-formats/ for a description of GitHub token formats.
-    token_re = re.compile('^(?:Token - )?(?P<tok>[0-9a-zA-Z_]{40}).*$')
+    token_re = re.compile('^(?:Token - )?(?P<tok>\w{40}).*$')
     github_token = None
 
     with open(github_token_file, 'rb') as token_stream:
@@ -536,8 +534,8 @@ def read_github_creds(github_token_file):
         if token_match:
             github_token = token_match.group('tok')
         else:
-            click.echo('Github token found in file "{}" does not match expected form, got: {}'
-                       .format(github_token_file, token_data), err=True)
+            click.echo('No Github token found in file "{}"'
+                       .format(github_token_file), err=True)
 
     return github_token
 
