@@ -12,15 +12,15 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <bsoncxx/builder/core.hpp>
-
 #include <cstring>
 
+#include <bsoncxx/builder/core.hpp>
 #include <bsoncxx/exception/error_code.hpp>
 #include <bsoncxx/exception/exception.hpp>
 #include <bsoncxx/private/itoa.hh>
 #include <bsoncxx/private/libbson.hh>
 #include <bsoncxx/private/stack.hh>
+#include <bsoncxx/private/suppress_deprecation_warnings.hh>
 #include <bsoncxx/stdx/make_unique.hpp>
 #include <bsoncxx/stdx/string_view.hpp>
 #include <bsoncxx/string/to_string.hpp>
@@ -287,7 +287,7 @@ core& core::append(const types::b_double& value) {
     return *this;
 }
 
-core& core::append(const types::b_utf8& value) {
+core& core::append(const types::b_string& value) {
     stdx::string_view key = _impl->next_key();
 
     if (!bson_append_utf8(_impl->back(),
@@ -295,7 +295,7 @@ core& core::append(const types::b_utf8& value) {
                           static_cast<std::int32_t>(key.length()),
                           value.value.data(),
                           static_cast<std::int32_t>(value.value.length()))) {
-        throw bsoncxx::exception{error_code::k_cannot_append_utf8};
+        throw bsoncxx::exception{error_code::k_cannot_append_string};
     }
 
     return *this;
@@ -544,13 +544,13 @@ core& core::append(const types::b_maxkey&) {
 }
 
 core& core::append(std::string str) {
-    append(types::b_utf8{std::move(str)});
+    append(types::b_string{std::move(str)});
 
     return *this;
 }
 
 core& core::append(stdx::string_view str) {
-    append(types::b_utf8{std::move(str)});
+    append(types::b_string{std::move(str)});
 
     return *this;
 }
