@@ -12,12 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <bsoncxx/document/value.hpp>
-
-#include <cstdlib>
-#include <cstring>
 #include <utility>
 
+#include <bsoncxx/document/value.hpp>
+
 #include <bsoncxx/config/private/prelude.hh>
 
 namespace bsoncxx {
@@ -50,6 +48,43 @@ value& value::operator=(const value& rhs) {
     return *this;
 }
 
+document::view::const_iterator value::cbegin() const {
+    return this->view().cbegin();
+}
+
+document::view::const_iterator value::cend() const {
+    return this->view().cend();
+}
+
+document::view::const_iterator value::begin() const {
+    return cbegin();
+}
+
+document::view::const_iterator value::end() const {
+    return cend();
+}
+
+document::view::const_iterator value::find(stdx::string_view key) const {
+    return this->view().find(key);
+}
+
+element value::operator[](stdx::string_view key) const {
+    auto view = this->view();
+    return view[key];
+}
+
+const std::uint8_t* value::data() const {
+    return _data.get();
+}
+
+std::size_t value::length() const {
+    return _length;
+}
+
+bool value::empty() const {
+    return _length == 5;
+}
+
 value::unique_ptr_type value::release() {
     _length = 0;
     return std::move(_data);
