@@ -12,15 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <bsoncxx/types/bson_value/view.hpp>
-
 #include <cstdlib>
-#include <cstring>
 
 #include <bsoncxx/exception/error_code.hpp>
 #include <bsoncxx/exception/exception.hpp>
-#include <bsoncxx/json.hpp>
 #include <bsoncxx/private/libbson.hh>
+#include <bsoncxx/private/suppress_deprecation_warnings.hh>
+#include <bsoncxx/types/bson_value/view.hpp>
 #include <bsoncxx/types/private/convert.hh>
 
 #include <bsoncxx/config/private/prelude.hh>
@@ -47,16 +45,16 @@ view::view() noexcept : view(nullptr) {}
 // so we can't rely on automatic noexcept propagation. It really is though, so it is OK.
 #if !defined(BSONCXX_POLY_USE_BOOST)
 #define BSONCXX_ENUM(name, val)                                                                \
-    view::view(b_##name value) noexcept : _type(static_cast<bsoncxx::type>(val)),              \
-                                          _b_##name(std::move(value)) {                        \
+    view::view(b_##name value) noexcept                                                        \
+        : _type(static_cast<bsoncxx::type>(val)), _b_##name(std::move(value)) {                \
         static_assert(std::is_nothrow_copy_constructible<b_##name>::value, "Copy may throw");  \
         static_assert(std::is_nothrow_copy_assignable<b_##name>::value, "Copy may throw");     \
         static_assert(std::is_nothrow_destructible<b_##name>::value, "Destruction may throw"); \
     }
 #else
 #define BSONCXX_ENUM(name, val)                                                                \
-    view::view(b_##name value) noexcept : _type(static_cast<bsoncxx::type>(val)),              \
-                                          _b_##name(std::move(value)) {                        \
+    view::view(b_##name value) noexcept                                                        \
+        : _type(static_cast<bsoncxx::type>(val)), _b_##name(std::move(value)) {                \
         static_assert(std::is_nothrow_destructible<b_##name>::value, "Destruction may throw"); \
     }
 #endif
@@ -91,6 +89,8 @@ view& view::operator=(const view& rhs) noexcept {
         break;
 #include <bsoncxx/enums/type.hpp>
 #undef BSONCXX_ENUM
+        default:
+            break;
     }
 
     _type = rhs._type;
@@ -113,6 +113,11 @@ bsoncxx::type view::type() const {
 #include <bsoncxx/enums/type.hpp>
 #undef BSONCXX_ENUM
 
+const types::b_string& view::get_utf8() const {
+    BSONCXX_TYPE_CHECK(string);
+    return _b_string;
+}
+
 view::view(const std::uint8_t* raw,
            std::uint32_t length,
            std::uint32_t offset,
