@@ -12,16 +12,16 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <mongocxx/client.hpp>
-
 #include <bsoncxx/builder/basic/kvp.hpp>
 #include <bsoncxx/stdx/make_unique.hpp>
+#include <mongocxx/client.hpp>
 #include <mongocxx/exception/error_code.hpp>
 #include <mongocxx/exception/logic_error.hpp>
 #include <mongocxx/exception/operation_exception.hpp>
 #include <mongocxx/exception/private/mongoc_error.hh>
 #include <mongocxx/options/auto_encryption.hpp>
 #include <mongocxx/options/private/apm.hh>
+#include <mongocxx/options/private/server_api.hh>
 #include <mongocxx/options/private/ssl.hh>
 #include <mongocxx/private/client.hh>
 #include <mongocxx/private/client_session.hh>
@@ -111,6 +111,19 @@ client::client(const class uri& uri, const options::client& options) {
         }
     }
 
+    if (options.server_api_opts()) {
+        const auto& server_api_opts = *options.server_api_opts();
+        auto mongoc_server_api_opts = options::make_server_api(server_api_opts);
+
+        bson_error_t error;
+        auto result = libmongoc::client_set_server_api(
+            _get_impl().client_t, mongoc_server_api_opts.get(), &error);
+
+        if (!result) {
+            throw_exception<operation_exception>(error);
+        }
+    }
+
 #if defined(MONGOCXX_ENABLE_SSL) && defined(MONGOC_ENABLE_SSL)
     if (options.tls_opts()) {
         auto mongoc_opts = options::make_tls_opts(*options.tls_opts());
@@ -195,9 +208,9 @@ cursor client::list_databases(const client_session& session) const {
     return libmongoc::client_find_databases_with_opts(_get_impl().client_t, options_bson.bson());
 }
 
-cursor client::list_databases(const bsoncxx::document::view_or_value filter) const {
-    scoped_bson_t filter_bson{filter.view()};
-    return libmongoc::client_find_databases_with_opts(_get_impl().client_t, filter_bson.bson());
+cursor client::list_databases(const bsoncxx::document::view_or_value opts) const {
+    scoped_bson_t opts_bson{opts.view()};
+    return libmongoc::client_find_databases_with_opts(_get_impl().client_t, opts_bson.bson());
 }
 
 std::vector<std::string> client::list_database_names(
