@@ -12,9 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include <bsoncxx/array/view_or_value.hpp>
 #include <mongocxx/options/find_one_and_update.hpp>
 
-#include <bsoncxx/array/view_or_value.hpp>
 #include <mongocxx/config/private/prelude.hh>
 
 namespace mongocxx {
@@ -37,6 +37,17 @@ find_one_and_update& find_one_and_update::hint(class hint index_hint) {
     return *this;
 }
 
+find_one_and_update& find_one_and_update::let(bsoncxx::document::view_or_value let) {
+    _let = let;
+    return *this;
+}
+
+find_one_and_update& find_one_and_update::comment(
+    bsoncxx::types::bson_value::view_or_value comment) {
+    _comment = std::move(comment);
+    return *this;
+}
+
 find_one_and_update& find_one_and_update::max_time(std::chrono::milliseconds max_time) {
     _max_time = std::move(max_time);
     return *this;
@@ -80,6 +91,15 @@ const stdx::optional<class hint>& find_one_and_update::hint() const {
     return _hint;
 }
 
+const stdx::optional<bsoncxx::document::view_or_value> find_one_and_update::let() const {
+    return _let;
+}
+
+const stdx::optional<bsoncxx::types::bson_value::view_or_value> find_one_and_update::comment()
+    const {
+    return _comment;
+}
+
 const stdx::optional<std::chrono::milliseconds>& find_one_and_update::max_time() const {
     return _max_time;
 }
