@@ -1,4 +1,4 @@
-// Copyright 2014 MongoDB Inc.
+// Copyright 2014-present MongoDB Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,10 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "helpers.hpp"
-
 #include <set>
 
+#include "helpers.hpp"
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/string/to_string.hpp>
 #include <bsoncxx/test_util/catch.hh>
@@ -75,7 +74,7 @@ TEST_CASE("A default constructed database cannot perform operations", "[database
 TEST_CASE("mongocxx::database copy constructor", "[database]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
 
     SECTION("constructing from valid") {
         database database_a = client["a"];
@@ -94,7 +93,7 @@ TEST_CASE("mongocxx::database copy constructor", "[database]") {
 TEST_CASE("mongocxx::database copy assignment operator", "[database]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
 
     SECTION("assigning valid to valid") {
         database database_a = client["a1"];
@@ -134,7 +133,7 @@ TEST_CASE("A database", "[database]") {
 
     instance::current();
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
 
     SECTION("is created by a client") {
         bool called = false;
@@ -181,7 +180,7 @@ TEST_CASE("A database", "[database]") {
         database_destroy->interpose([&](mongoc_database_t*) { destroy_called = true; });
 
         {
-            client mongo_client{uri{}};
+            client mongo_client{uri{}, test_util::add_test_server_api()};
             database a = mongo_client[database_name];
 
             database b{std::move(a)};
@@ -322,7 +321,7 @@ TEST_CASE("A database", "[database]") {
 TEST_CASE("Database integration tests", "[database]") {
     instance::current();
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
     stdx::string_view database_name{"database"};
     database database = mongo_client[database_name];
 
@@ -346,19 +345,17 @@ TEST_CASE("Database integration tests", "[database]") {
             }
 
             auto session1 = mongo_client.start_session();
-            auto session2 = mongo_client.start_session();
 
             pipeline.list_local_sessions({});
-            pipeline.limit(2);
+            pipeline.limit(1);
             pipeline.add_fields(make_document(kvp("name", "Jane")));
             pipeline.project(make_document(kvp("name", 1)));
 
             auto cursor = database.aggregate(pipeline);
             auto results = get_results(std::move(cursor));
 
-            REQUIRE(results.size() == 2);
+            REQUIRE(results.size() == 1);
             REQUIRE(results[0].view()["name"].get_string().value == stdx::string_view("Jane"));
-            REQUIRE(results[1].view()["name"].get_string().value == stdx::string_view("Jane"));
         }
     }
 
@@ -495,4 +492,93 @@ TEST_CASE("Database integration tests", "[database]") {
     }
 }
 
+// As C++11 lacks generic lambdas, and "ordinary" templates can't appear at block scope,
+// we'll have to define our helper for the serviceId tests here. This implementation would
+// be more straightforward in newer versions of C++ (C++14 and on allow generic lambdas),
+// see CXX-2350 (migration to more recent C++ standards); our C++17 optional<> implementation
+// also has a few inconsistencies with the standard, which appear to vary across platforms.
+template <typename EventT>
+struct check_service_id {
+    const bool expect_service_id;
+
+    check_service_id(const bool expect_service_id) : expect_service_id(expect_service_id) {}
+
+    void operator()(const EventT& event) {
+        INFO("checking for service_id()")
+        CAPTURE(event.command_name(), expect_service_id);
+
+        auto service_id = event.service_id();
+
+        if (expect_service_id)
+            CHECK(service_id);
+        else
+            CHECK_FALSE(service_id);
+    }
+};
+
+TEST_CASE("serviceId presence depends on load-balancing mode") {
+    // Repeat this test case for a situation in which service_id should and should not be returned:
+    auto expect_service_id = GENERATE(true, false);
+
+    instance::current();
+
+    auto client_opts = test_util::add_test_server_api();
+
+    // Set Application Performance Monitoring options (APM):
+    mongocxx::options::apm apm_opts;
+    apm_opts.on_command_started(
+        check_service_id<mongocxx::events::command_started_event>{expect_service_id});
+    apm_opts.on_command_succeeded(
+        check_service_id<mongocxx::events::command_succeeded_event>{expect_service_id});
+    apm_opts.on_command_failed(
+        check_service_id<mongocxx::events::command_failed_event>{expect_service_id});
+
+    // Set up mocking for get_service_id events:
+    auto apm_command_started_get_service_id =
+        libmongoc::apm_command_started_get_service_id.create_instance();
+    auto apm_command_succeeded_get_service_id =
+        libmongoc::apm_command_succeeded_get_service_id.create_instance();
+    auto apm_command_failed_get_service_id =
+        libmongoc::apm_command_failed_get_service_id.create_instance();
+
+    // Set up mocked functions that DO emit a service_id:
+    if (expect_service_id) {
+        // Return a bson_oid_t with data where the service_id has some value:
+        const auto make_service_id_bson_oid_t = [](const void*) -> const bson_oid_t* {
+            static bson_oid_t tmp = {0x65};
+            return &tmp;
+        };
+
+        // Add forever() so that the mock function also extends to endSession:
+        apm_command_started_get_service_id->interpose(make_service_id_bson_oid_t).forever();
+        apm_command_succeeded_get_service_id->interpose(make_service_id_bson_oid_t).forever();
+        apm_command_failed_get_service_id->interpose(make_service_id_bson_oid_t).forever();
+    }
+
+    // Set up mocked functions that DO NOT emit a service_id:
+    if (!expect_service_id) {
+        auto make_empty_bson_oid_t = [](const void*) -> bson_oid_t* { return nullptr; };
+
+        apm_command_started_get_service_id->interpose(make_empty_bson_oid_t).forever();
+        apm_command_succeeded_get_service_id->interpose(make_empty_bson_oid_t).forever();
+        apm_command_failed_get_service_id->interpose(make_empty_bson_oid_t).forever();
+    }
+
+    // Set up our connection:
+    client_opts.apm_opts(apm_opts);
+    // Adding ?loadBalanced=true results in an error in the C driver because
+    // the initial hello response from the server does not include serviceId.
+    client mongo_client(uri("mongodb://localhost:27017/"), client_opts);
+    stdx::string_view database_name{"database"};
+    database database = mongo_client[database_name];
+
+    // Run a command, triggering start and completion events:
+    auto cmd = make_document(kvp("ping", 1));
+    database.run_command(cmd.view());
+
+    // Attempt to trigger failure:
+    cmd = make_document(kvp("some_sort_of_invalid_command_that_should_never_happen", 1));
+    CHECK_THROWS(database.run_command(cmd.view()));
+}
+
 }  // namespace
