@@ -13,6 +13,15 @@
 // limitations under the License.
 
 #include <algorithm>
+#include <chrono>
+#include <cstdint>
+#include <cstdlib>
+#include <fstream>
+#include <functional>
+#include <numeric>
+#include <sstream>
+#include <vector>
+
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/builder/basic/kvp.hpp>
 #include <bsoncxx/document/value.hpp>
@@ -20,11 +29,6 @@
 #include <bsoncxx/test_util/catch.hh>
 #include <bsoncxx/types.hpp>
 #include <bsoncxx/types/bson_value/view.hpp>
-#include <chrono>
-#include <cstdint>
-#include <cstdlib>
-#include <fstream>
-#include <functional>
 #include <mongocxx/client.hpp>
 #include <mongocxx/database.hpp>
 #include <mongocxx/exception/gridfs_exception.hpp>
@@ -34,10 +38,8 @@
 #include <mongocxx/options/find.hpp>
 #include <mongocxx/options/gridfs/upload.hpp>
 #include <mongocxx/options/index.hpp>
+#include <mongocxx/test_util/client_helpers.hh>
 #include <mongocxx/uri.hpp>
-#include <numeric>
-#include <sstream>
-#include <vector>
 
 namespace {
 using namespace mongocxx;
@@ -61,7 +63,7 @@ void validate_gridfs_file(database db,
     REQUIRE(static_cast<std::size_t>(files_doc->view()["length"].get_int64().value) ==
             expected_contents.size());
     REQUIRE(files_doc->view()["chunkSize"].get_int32().value == expected_chunk_size);
-    REQUIRE(files_doc->view()["filename"].get_utf8().value ==
+    REQUIRE(files_doc->view()["filename"].get_string().value ==
             stdx::string_view(expected_file_name));
 
     // md5 is deprecated in GridFS, we don't include it:
@@ -119,7 +121,7 @@ void validate_gridfs_file(
     REQUIRE(files_doc->view()["_id"].get_oid() == id.get_oid());
     REQUIRE(files_doc->view()["length"].get_int64().value == expected_length);
     REQUIRE(files_doc->view()["chunkSize"].get_int32().value == expected_chunk_size);
-    REQUIRE(files_doc->view()["filename"].get_utf8().value ==
+    REQUIRE(files_doc->view()["filename"].get_string().value ==
             stdx::string_view(expected_file_name));
 
     std::size_t i = 0;
@@ -227,7 +229,7 @@ TEST_CASE("mongocxx::gridfs::bucket default constructor makes invalid bucket", "
 TEST_CASE("mongocxx::gridfs::bucket copy constructor", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_bucket_copy_constructor"];
 
     SECTION("constructing from valid") {
@@ -247,7 +249,7 @@ TEST_CASE("mongocxx::gridfs::bucket copy constructor", "[gridfs::bucket]") {
 TEST_CASE("mongocxx::gridfs::bucket copy assignment operator", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_bucket_copy_assignment"];
 
     SECTION("assigning valid to valid") {
@@ -284,7 +286,7 @@ TEST_CASE("mongocxx::gridfs::bucket copy assignment operator", "[gridfs::bucket]
 TEST_CASE("database::gridfs_bucket() throws error when options are invalid", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_bucket_error_invalid_options"];
 
     options::gridfs::bucket bucket_options;
@@ -306,7 +308,7 @@ TEST_CASE("database::gridfs_bucket() throws error when options are invalid", "[g
 TEST_CASE("uploading throws error when options are invalid", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_error_invalid_options"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -343,7 +345,7 @@ TEST_CASE("uploading throws error when options are invalid", "[gridfs::bucket]")
 TEST_CASE("downloading throws error when files document is corrupt", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_files_doc_corrupt"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -400,7 +402,7 @@ TEST_CASE("downloading throws error when files document is corrupt", "[gridfs::b
         run_test();
     }
     SECTION("chunk size of wrong type") {
-        chunk_size = bsoncxx::types::bson_value::view{bsoncxx::types::b_utf8{"invalid"}};
+        chunk_size = bsoncxx::types::bson_value::view{bsoncxx::types::b_string{"invalid"}};
         run_test();
     }
     SECTION("missing chunk size") {
@@ -414,7 +416,7 @@ TEST_CASE("downloading throws error when files document is corrupt", "[gridfs::b
         run_test();
     }
     SECTION("invalid length") {
-        length = bsoncxx::types::bson_value::view{bsoncxx::types::b_utf8{"invalid"}};
+        length = bsoncxx::types::bson_value::view{bsoncxx::types::b_string{"invalid"}};
         run_test();
     }
     SECTION("missing length") {
@@ -440,7 +442,7 @@ TEST_CASE("downloading throws error when files document is corrupt", "[gridfs::b
 TEST_CASE("downloading throws error when chunks document is corrupt", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_chunk_doc_corrupt"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -521,7 +523,7 @@ TEST_CASE("downloading throws error when chunks document is corrupt", "[gridfs::
 TEST_CASE("mongocxx::gridfs::downloader::read with arbitrary sizes", "[gridfs::downloader]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_download_read_test"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -600,7 +602,7 @@ TEST_CASE("mongocxx::gridfs::downloader::read with arbitrary sizes", "[gridfs::d
 TEST_CASE("mongocxx::gridfs::uploader::abort works", "[gridfs::uploader]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_abort_test"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -633,7 +635,7 @@ TEST_CASE("mongocxx::gridfs::uploader::abort works", "[gridfs::uploader]") {
 TEST_CASE("mongocxx::gridfs::uploader::write with arbitrary sizes", "[gridfs::uploader]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_write_test"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -648,7 +650,6 @@ TEST_CASE("mongocxx::gridfs::uploader::write with arbitrary sizes", "[gridfs::up
     std::int32_t write_size;
 
     auto run_test = [&]() {
-
         std::vector<std::uint8_t> bytes;
 
         // Populate the vector with arbitrary values.
@@ -708,7 +709,7 @@ TEST_CASE("mongocxx::gridfs::uploader::write with arbitrary sizes", "[gridfs::up
 TEST_CASE("gridfs upload/download round trip", "[gridfs::uploader] [gridfs::downloader]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_download_round_trip_test"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -741,7 +742,7 @@ TEST_CASE("gridfs upload/download round trip", "[gridfs::uploader] [gridfs::down
 TEST_CASE("gridfs::bucket::open_upload_stream_with_id works", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     auto db = client["gridfs_bucket_open_upload_stream_with_id"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -768,7 +769,7 @@ TEST_CASE("gridfs::bucket::open_upload_stream_with_id works", "[gridfs::bucket]"
 TEST_CASE("gridfs::bucket::upload_from_stream works", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     auto db = client["gridfs_bucket_upload_from_stream_works"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -809,7 +810,7 @@ TEST_CASE("gridfs::bucket::upload_from_stream doesn't infinite loop when passed
           "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_from_stream_no_infinite_loop_ifstream"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -828,7 +829,7 @@ TEST_CASE("gridfs::bucket::upload_from_stream doesn't infinite loop when passed
 TEST_CASE("gridfs upload_from_stream aborts on failure", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_from_stream_abort_on_failure"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -849,31 +850,127 @@ TEST_CASE("gridfs upload_from_stream aborts on failure", "[gridfs::bucket]") {
 TEST_CASE("gridfs::bucket::download_to_stream works", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_download_to_stream_works"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
     db["fs.files"].drop();
     db["fs.chunks"].drop();
 
-    std::int64_t length = 10;
+    std::int64_t length = 19;  // length is the length of the test file in bytes.
     std::int32_t chunk_size = 4;
     bsoncxx::types::bson_value::view id{bsoncxx::types::b_oid{bsoncxx::oid{}}};
-    auto expected_bytes = manual_gridfs_initialize(db, length, chunk_size, id);
-
+    auto file_bytes = manual_gridfs_initialize(db, length, chunk_size, id);
     std::ostringstream os;
-    bucket.download_to_stream(id, &os);
 
-    auto str = os.str();
-    std::vector<std::uint8_t> actual_bytes{str.begin(), str.end()};
+    SECTION("download complete file") {
+        bucket.download_to_stream(id, &os);
 
-    REQUIRE(expected_bytes == actual_bytes);
+        auto str = os.str();
+        std::vector<std::uint8_t> actual_bytes{str.begin(), str.end()};
+
+        REQUIRE(file_bytes == actual_bytes);
+    }
+
+    SECTION("download complete file with start/end offsets") {
+        bucket.download_to_stream(id, &os, 0, static_cast<size_t>(length));
+
+        auto str = os.str();
+        std::vector<std::uint8_t> actual_bytes{str.begin(), str.end()};
+
+        REQUIRE(file_bytes == actual_bytes);
+    }
+
+    SECTION("download partial file") {
+        auto check_downloaded_content = [&](std::size_t start, std::size_t end) {
+            bucket.download_to_stream(id, &os, start, end);
+
+            auto str = os.str();
+            std::vector<std::uint8_t> actual_bytes{str.begin(), str.end()};
+            REQUIRE(actual_bytes.size() == end - start);
+            const std::vector<std::uint8_t> expected_bytes{
+                file_bytes.begin() + static_cast<std::vector<uint8_t>::difference_type>(start),
+                file_bytes.begin() + static_cast<std::vector<uint8_t>::difference_type>(end)};
+            REQUIRE(expected_bytes == actual_bytes);
+        };
+
+        SECTION("whole file") {
+            check_downloaded_content(0, static_cast<std::size_t>(length));
+        }
+
+        SECTION("no data") {
+            check_downloaded_content(0, 0);
+        }
+
+        SECTION("at file start") {
+            SECTION("partial chunk") {
+                check_downloaded_content(0, static_cast<std::size_t>(chunk_size - 1));
+            }
+
+            SECTION("complete chunk") {
+                check_downloaded_content(0, static_cast<std::size_t>(chunk_size));
+            }
+
+            SECTION("across chunks") {
+                check_downloaded_content(0, static_cast<std::size_t>(2 * chunk_size));
+            }
+        }
+
+        SECTION("middle of the file") {
+            SECTION("partial chunk") {
+                const auto start = chunk_size + chunk_size / 2;
+                const auto end = start + 1;
+                check_downloaded_content(static_cast<std::size_t>(start),
+                                         static_cast<std::size_t>(end));
+            }
+
+            SECTION("complete chunk") {
+                const auto start = chunk_size;
+                const auto end = start + chunk_size;
+                check_downloaded_content(static_cast<std::size_t>(start),
+                                         static_cast<std::size_t>(end));
+            }
+
+            SECTION("across 2 chunks") {
+                const auto start = chunk_size / 2;
+                const auto end = start + chunk_size;
+                check_downloaded_content(static_cast<std::size_t>(start),
+                                         static_cast<std::size_t>(end));
+            }
+
+            SECTION("across 3 chunks") {
+                const auto start = chunk_size / 2;
+                const auto end = start + 2 * chunk_size;
+                check_downloaded_content(static_cast<std::size_t>(start),
+                                         static_cast<std::size_t>(end));
+            }
+        }
+
+        SECTION("at file end") {
+            const auto last_chunk_start = chunk_size * (length / chunk_size);
+            SECTION("across chunks") {
+                check_downloaded_content(
+                    static_cast<std::size_t>(last_chunk_start - (2 * chunk_size) + 1),
+                    static_cast<std::size_t>(length));
+            }
+
+            SECTION("partial chunk") {
+                check_downloaded_content(static_cast<std::size_t>(last_chunk_start + 1),
+                                         static_cast<std::size_t>(length));
+            }
+
+            SECTION("complete chunk") {
+                check_downloaded_content(static_cast<std::size_t>(last_chunk_start),
+                                         static_cast<std::size_t>(length));
+            }
+        }
+    }
 }
 
 TEST_CASE("gridfs::bucket::delete_file works", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_delete_file_works"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -894,7 +991,7 @@ TEST_CASE("gridfs::bucket::delete_file works", "[gridfs::bucket]") {
 TEST_CASE("gridfs::bucket::find works", "[gridfs::bucket]") {
     instance::current();
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_find_works"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -940,7 +1037,7 @@ TEST_CASE("gridfs upload large file", "[gridfs::bucket]") {
         return;
     }
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_upload_large_file"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -997,7 +1094,7 @@ TEST_CASE("gridfs download large file", "[gridfs::bucket]") {
         return;
     }
 
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     database db = client["gridfs_download_large_file"];
     gridfs::bucket bucket = db.gridfs_bucket();
 
@@ -1070,7 +1167,7 @@ TEST_CASE("gridfs does not create additional indexes", "[gridfs::uploader] [grid
     using bsoncxx::builder::basic::sub_array;
 
     instance::current();
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     auto db_name = _gen_database_name("gridfs_index_creation");
     database db = client[db_name];
 
