@@ -13,10 +13,7 @@
 // limitations under the License.
 
 #include <bsoncxx/builder/basic/document.hpp>
-#include <bsoncxx/builder/concatenate.hpp>
-#include <bsoncxx/document/element.hpp>
 #include <bsoncxx/document/view.hpp>
-#include <bsoncxx/document/view_or_value.hpp>
 #include <bsoncxx/json.hpp>
 #include <bsoncxx/string/to_string.hpp>
 #include <bsoncxx/test_util/catch.hh>
@@ -27,9 +24,9 @@ namespace {
 using namespace mongocxx;
 using namespace bsoncxx;
 
+using bsoncxx::builder::concatenate;
 using bsoncxx::builder::basic::kvp;
 using bsoncxx::builder::basic::make_document;
-using bsoncxx::builder::concatenate;
 
 TEST_CASE("Hint", "[hint]") {
     instance::current();
@@ -40,8 +37,8 @@ TEST_CASE("Hint", "[hint]") {
 
         SECTION("Returns correct value from to_value") {
             types::bson_value::view val = index_hint.to_value();
-            REQUIRE(val.type() == type::k_utf8);
-            REQUIRE(bsoncxx::string::to_string(val.get_utf8().value) == index_name);
+            REQUIRE(val.type() == type::k_string);
+            REQUIRE(bsoncxx::string::to_string(val.get_string().value) == index_name);
         }
 
         SECTION("Compares equal to matching index name") {
