@@ -13,7 +13,6 @@
 // limitations under the License.
 
 #include "helpers.hpp"
-
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/document/element.hpp>
 #include <bsoncxx/string/to_string.hpp>
@@ -77,13 +76,13 @@ TEST_CASE("validation_criteria can be exported to a document", "[validation_crit
 
     ele = criteria_view["validationLevel"];
     REQUIRE(ele);
-    REQUIRE(ele.type() == type::k_utf8);
-    REQUIRE(bsoncxx::string::to_string(ele.get_utf8().value) == "strict");
+    REQUIRE(ele.type() == type::k_string);
+    REQUIRE(bsoncxx::string::to_string(ele.get_string().value) == "strict");
 
     ele = criteria_view["validationAction"];
     REQUIRE(ele);
-    REQUIRE(ele.type() == type::k_utf8);
-    REQUIRE(bsoncxx::string::to_string(ele.get_utf8().value) == "warn");
+    REQUIRE(ele.type() == type::k_string);
+    REQUIRE(bsoncxx::string::to_string(ele.get_string().value) == "warn");
 
     ele = criteria_view["validator"];
     REQUIRE(ele);
